@@ -2,9 +2,9 @@ import re
 from copy import deepcopy
 from xml.dom import minidom, getDOMImplementation
 
-from .util import openAnything, ReadableException
-from .pathfeatures import PathDescription, Join, SortOrder, SortOrderList
-from .model import Column, Class
+from intermine.util import openAnything, ReadableException
+from intermine.pathfeatures import PathDescription, Join, SortOrder, SortOrderList
+from intermine.model import Column, Class
 import constraints
 
 """
@@ -276,6 +276,11 @@ class Query(object):
     """
 
     SO_SPLIT_PATTERN = re.compile("\s*(asc|desc)\s*", re.I)
+    LOGIC_SPLIT_PATTERN = re.compile("\s*(?:and|or|\(|\))\s*", re.I)
+    TRAILING_OP_PATTERN = re.compile("\s*(and|or)\s*$", re.I)
+    LEADING_OP_PATTERN = re.compile("^\s*(and|or)\s*", re.I)
+    LOGIC_OPS = ["and", "or"]
+    LOGIC_PRODUCT = [(x, y) for x in LOGIC_OPS for y in LOGIC_OPS]
 
     def __init__(self, model, service=None, validate=True, root=None):
         """
@@ -423,29 +428,31 @@ class Query(object):
         if q.getAttribute('constraintLogic') is not None:
             logic = q.getAttribute('constraintLogic')
             used_codes = set(obj.constraint_dict.keys())
-            logic_codes = set(re.split("\s*(?:and|or|\(|\))\s*", logic, flags=re.I))
+            logic_codes = set(Query.LOGIC_SPLIT_PATTERN.split(logic))
             if "" in logic_codes:
                 logic_codes.remove("")
             irrelevant_codes = logic_codes - used_codes
             for c in irrelevant_codes:
-                logic = re.sub("((and|or)\s+)?" + c + "(\s+(and|or))?", "", logic)
+                pattern = re.compile("((and|or)\s+)?\\b" + c + "\\b(\s+(and|or))?", re.I)
+                logic = pattern.sub("", logic)
             # Remove empty groups
             logic = re.sub("\(\s*\)", "", logic)
             # Remove trailing and leading opertators
-            logic = re.sub("\s*(and|or)\s*$", "", logic, flags=re.I)
-            logic = re.sub("^\s*(and|or)\s*", "", logic, flags=re.I)
-            ops = ["and", "or"]
-            for left, right in [(x, y) for x in ops for y in ops]:
-                repl = left if left == right else "and"
-                logic = re.sub(left + "\s*" + right, repl, logic, flags=re.I)
-                if left != right:
-                    logic = re.sub(right + "\s*" + left, repl, logic, flags=re.I)
+            logic = Query.LEADING_OP_PATTERN.sub("", logic)
+            logic = Query.TRAILING_OP_PATTERN.sub("", logic)
+            for left, right in Query.LOGIC_PRODUCT:
+                if left == right:
+                    repl = left
+                else:
+                    repl = "and"
+                pattern = re.compile(left + "\s*" + right, re.I)
+                logic = pattern.sub(repl, logic)
             logic = logic.strip().lstrip()
             try: 
                 if len(logic) > 0:
                     obj.set_logic(logic)
-            except:
-                raise Exception("Error parsing " + q.getAttribute('constraintLogic') + " => " + repr(logic) + " with views: " + repr(used_codes))
+            except Exception, e:
+                raise Exception("Error parsing " + q.getAttribute('constraintLogic') + " => " + repr(logic) + " with views: " + repr(used_codes) + e.message)
 
         obj.verify()        
 
@@ -789,12 +796,16 @@ class Query(object):
 
         example::
             
-            query.add_path_description("Gene.symbol", "The symbol for this gene")
+            query.add_path_description("Gene.proteins.proteinDomains", "Protein Domain")
 
-        If you wish you can add annotations to your query that describe
-        what the component paths are and what they do - this is only really
-        useful if you plan to keep your query (perhaps as xml) or store it
-        as a template.
+        This allows you to alias the components of long paths to 
+        improve the way they display column headers in a variety of circumstances. 
+        In the above example, if the view included the unwieldy path
+        "Gene.proteins.proteinDomains.primaryIdentifier", it would (depending on the 
+        mine) be displayed as "Protein Domain > DB Identifer". These
+        setting are taken into account by the webservice when generating 
+        column headers for flat-file results with the columnheaders parameter given, and
+        always supplied when requesting jsontable results.
 
         @rtype: L{intermine.pathfeatures.PathDescription}
 
@@ -1047,7 +1058,10 @@ class Query(object):
 
     def first(self, row="rr", start=0):
         """Return the first result, or None if the results are empty"""
-        size = None if row == "jsonobjects" else 1
+        if row == "jsonobjects":
+            size = None
+        else:
+            size = 1
         try:
             return self.results(row, start=start, size=size).next()
         except StopIteration:
@@ -1363,8 +1377,8 @@ class Template(Query):
         for c in self.editable_constraints:
             if not c.switched_on: next
             for k, v in c.to_dict().items():
-                k = "extra" if k == "extraValue" else k
-                k = "constraint" if k == "path" else k
+                if k == "extraValue": k = "extra" 
+                if k == "path": k = "constraint"
                 p[k + str(i)] = v
             i += 1
         return p
